@@ -1,7 +1,7 @@
 //!
 //! Functionality for rendering to the screen or into textures.
 //!
-use crate::context::{consts, Context, DataType};
+use crate::context::{consts, DataType};
 use crate::core::*;
 
 ///
@@ -94,11 +94,11 @@ impl Screen {
     /// in the `render` closure to render something to the screen.
     /// Before writing, the screen is cleared based on the given clear state.
     ///
-    pub fn write<F: FnOnce() -> Result<()>>(
+    pub fn write<F: FnOnce() -> ThreeDResult<()>>(
         context: &Context,
         clear_state: ClearState,
         render: F,
-    ) -> Result<()> {
+    ) -> ThreeDResult<()> {
         context.bind_framebuffer(consts::DRAW_FRAMEBUFFER, None);
         clear(context, &clear_state);
         render()?;
@@ -108,7 +108,7 @@ impl Screen {
     ///
     /// Returns the RGBA color values from the screen as a list of bytes (one byte for each color channel).
     ///
-    pub fn read_color(context: &Context, viewport: Viewport) -> Result<Vec<u8>> {
+    pub fn read_color(context: &Context, viewport: Viewport) -> ThreeDResult<Vec<u8>> {
         let mut pixels = vec![0u8; viewport.width as usize * viewport.height as usize * 4];
         context.bind_framebuffer(consts::READ_FRAMEBUFFER, None);
         context.read_pixels_with_u8_data(
@@ -128,7 +128,7 @@ impl Screen {
     /// Only available on desktop.
     ///
     #[cfg(not(target_arch = "wasm32"))]
-    pub fn read_depth(context: &Context, viewport: Viewport) -> Result<Vec<f32>> {
+    pub fn read_depth(context: &Context, viewport: Viewport) -> ThreeDResult<Vec<f32>> {
         let mut pixels = vec![0f32; viewport.width as usize * viewport.height as usize];
         context.bind_framebuffer(consts::READ_FRAMEBUFFER, None);
         context.read_pixels_with_f32_data(
@@ -179,7 +179,7 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
         context: &Context,
         color_texture: &'a ColorTargetTexture2D<T>,
         depth_texture: &'b DepthTargetTexture2D,
-    ) -> Result<Self> {
+    ) -> ThreeDResult<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -192,11 +192,11 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
     /// Renders whatever rendered in the `render` closure into the textures defined at construction.
     /// Before writing, the textures are cleared based on the given clear state.
     ///
-    pub fn write<F: FnOnce() -> Result<()>>(
+    pub fn write<F: FnOnce() -> ThreeDResult<()>>(
         &self,
         clear_state: ClearState,
         render: F,
-    ) -> Result<()> {
+    ) -> ThreeDResult<()> {
         self.bind(consts::DRAW_FRAMEBUFFER)?;
         clear(
             &self.context,
@@ -224,32 +224,38 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
         destination: CopyDestination<T>,
         viewport: Viewport,
         write_mask: WriteMask,
-    ) -> Result<()> {
+    ) -> ThreeDResult<()> {
         let copy = || {
-            let effect = get_copy_effect(&self.context)?;
-            if let Some(tex) = self.color_texture {
-                effect.use_texture("colorMap", tex)?;
-            }
-            if let Some(tex) = self.depth_texture {
-                effect.use_texture("depthMap", tex)?;
-            }
-            effect.apply(
-                RenderStates {
-                    depth_test: DepthTest::Always,
-                    write_mask,
-                    ..Default::default()
-                },
-                viewport,
-            )?;
-            Ok(())
+            let fragment_shader_source = "
+            uniform sampler2D colorMap;
+            uniform sampler2D depthMap;
+            in vec2 uv;
+            layout (location = 0) out vec4 color;
+            void main()
+            {
+                color = texture(colorMap, uv);
+                gl_FragDepth = texture(depthMap, uv).r;
+            }";
+            self.context.effect(fragment_shader_source, |effect| {
+                if let Some(tex) = self.color_texture {
+                    effect.use_texture("colorMap", tex)?;
+                }
+                if let Some(tex) = self.depth_texture {
+                    effect.use_texture("depthMap", tex)?;
+                }
+                effect.apply(
+                    RenderStates {
+                        depth_test: DepthTest::Always,
+                        write_mask,
+                        ..Default::default()
+                    },
+                    viewport,
+                )
+            })
         };
         match destination {
-            CopyDestination::RenderTarget(other) => {
-                other.write(ClearState::none(), copy)?;
-            }
-            CopyDestination::Screen => {
-                Screen::write(&self.context, ClearState::none(), copy)?;
-            }
+            CopyDestination::RenderTarget(other) => other.write(ClearState::none(), copy),
+            CopyDestination::Screen => Screen::write(&self.context, ClearState::none(), copy),
             CopyDestination::ColorTexture(tex) => {
                 if self.color_texture.is_none() {
                     Err(CoreError::RenderTargetCopy(
@@ -257,7 +263,7 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
                         "depth".to_string(),
                     ))?;
                 }
-                tex.write(ClearState::none(), copy)?;
+                tex.write(ClearState::none(), copy)
             }
             CopyDestination::DepthTexture(tex) => {
                 if self.depth_texture.is_none() {
@@ -266,16 +272,15 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
                         "color".to_string(),
                     ))?;
                 }
-                tex.write(None, copy)?;
+                tex.write(None, copy)
             }
         }
-        Ok(())
     }
 
     pub(super) fn new_color(
         context: &Context,
         color_texture: &'a ColorTargetTexture2D<T>,
-    ) -> Result<Self> {
+    ) -> ThreeDResult<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -287,7 +292,7 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
     pub(super) fn new_depth(
         context: &Context,
         depth_texture: &'b DepthTargetTexture2D,
-    ) -> Result<Self> {
+    ) -> ThreeDResult<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -296,7 +301,7 @@ impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
         })
     }
 
-    pub(super) fn bind(&self, target: u32) -> Result<()> {
+    pub(super) fn bind(&self, target: u32) -> ThreeDResult<()> {
         self.context.bind_framebuffer(target, Some(&self.id));
         if let Some(tex) = self.color_texture {
             self.context.draw_buffers(&[consts::COLOR_ATTACHMENT0]);
@@ -338,7 +343,7 @@ impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
         context: &Context,
         color_texture: &'a ColorTargetTexture2DArray<T>,
         depth_texture: &'b DepthTargetTexture2DArray,
-    ) -> Result<Self> {
+    ) -> ThreeDResult<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -350,7 +355,7 @@ impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
     pub(super) fn new_color(
         context: &Context,
         color_texture: &'a ColorTargetTexture2DArray<T>,
-    ) -> Result<Self> {
+    ) -> ThreeDResult<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -362,7 +367,7 @@ impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
     pub(super) fn new_depth(
         context: &Context,
         depth_texture: &'b DepthTargetTexture2DArray,
-    ) -> Result<Self> {
+    ) -> ThreeDResult<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -383,8 +388,8 @@ impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
         color_layers: &[u32],
         depth_layer: u32,
         clear_state: ClearState,
-        render: impl FnOnce() -> Result<()>,
-    ) -> Result<()> {
+        render: impl FnOnce() -> ThreeDResult<()>,
+    ) -> ThreeDResult<()> {
         self.bind(Some(color_layers), Some(depth_layer))?;
         clear(
             &self.context,
@@ -414,26 +419,38 @@ impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
         destination: CopyDestination<T>,
         viewport: Viewport,
         write_mask: WriteMask,
-    ) -> Result<()> {
+    ) -> ThreeDResult<()> {
         let copy = || {
-            let effect = get_copy_array_effect(&self.context)?;
-            if let Some(tex) = self.color_texture {
-                effect.use_texture_array("colorMap", tex)?;
-                effect.use_uniform_int("colorLayer", &(color_layer as i32))?;
-            }
-            if let Some(tex) = self.depth_texture {
-                effect.use_texture_array("depthMap", tex)?;
-                effect.use_uniform_int("depthLayer", &(depth_layer as i32))?;
-            }
-            effect.apply(
-                RenderStates {
-                    depth_test: DepthTest::Always,
-                    write_mask,
-                    ..Default::default()
-                },
-                viewport,
-            )?;
-            Ok(())
+            let fragment_shader_source = "
+            uniform sampler2DArray colorMap;
+            uniform sampler2DArray depthMap;
+            uniform int colorLayer;
+            uniform int depthLayer;
+            in vec2 uv;
+            layout (location = 0) out vec4 color;
+            void main()
+            {
+                color = texture(colorMap, vec3(uv, colorLayer));
+                gl_FragDepth = texture(depthMap, vec3(uv, depthLayer)).r;
+            }";
+            self.context.effect(fragment_shader_source, |effect| {
+                if let Some(tex) = self.color_texture {
+                    effect.use_texture_array("colorMap", tex)?;
+                    effect.use_uniform_int("colorLayer", &(color_layer as i32))?;
+                }
+                if let Some(tex) = self.depth_texture {
+                    effect.use_texture_array("depthMap", tex)?;
+                    effect.use_uniform_int("depthLayer", &(depth_layer as i32))?;
+                }
+                effect.apply(
+                    RenderStates {
+                        depth_test: DepthTest::Always,
+                        write_mask,
+                        ..Default::default()
+                    },
+                    viewport,
+                )
+            })
         };
         match destination {
             CopyDestination::RenderTarget(other) => {
@@ -464,7 +481,7 @@ impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
         Ok(())
     }
 
-    fn bind(&self, color_layers: Option<&[u32]>, depth_layer: Option<u32>) -> Result<()> {
+    fn bind(&self, color_layers: Option<&[u32]>, depth_layer: Option<u32>) -> ThreeDResult<()> {
         self.context
             .bind_framebuffer(consts::DRAW_FRAMEBUFFER, Some(&self.id));
         if let Some(color_texture) = self.color_texture {
@@ -496,14 +513,14 @@ impl<T: TextureDataType> Drop for RenderTargetArray<'_, '_, T> {
     }
 }
 
-fn new_framebuffer(context: &Context) -> Result<crate::context::Framebuffer> {
+fn new_framebuffer(context: &Context) -> ThreeDResult<crate::context::Framebuffer> {
     Ok(context
         .create_framebuffer()
         .ok_or(CoreError::RenderTargetCreation)?)
 }
 
 #[cfg(feature = "debug")]
-fn check(context: &Context) -> Result<()> {
+fn check(context: &Context) -> ThreeDResult<()> {
     context
         .check_framebuffer_status()
         .or_else(|status| Err(CoreError::RenderTargetCreation))
@@ -545,49 +562,3 @@ fn clear(context: &Context, clear_state: &ClearState) {
         }
     });
 }
-
-fn get_copy_effect(context: &Context) -> Result<&ImageEffect> {
-    unsafe {
-        static mut COPY_EFFECT: Option<ImageEffect> = None;
-        if COPY_EFFECT.is_none() {
-            COPY_EFFECT = Some(ImageEffect::new(
-                context,
-                &"
-                uniform sampler2D colorMap;
-                uniform sampler2D depthMap;
-                in vec2 uv;
-                layout (location = 0) out vec4 color;
-                void main()
-                {
-                    color = texture(colorMap, uv);
-                    gl_FragDepth = texture(depthMap, uv).r;
-                }",
-            )?);
-        }
-        Ok(COPY_EFFECT.as_ref().unwrap())
-    }
-}
-
-fn get_copy_array_effect(context: &Context) -> Result<&ImageEffect> {
-    unsafe {
-        static mut COPY_EFFECT: Option<ImageEffect> = None;
-        if COPY_EFFECT.is_none() {
-            COPY_EFFECT = Some(ImageEffect::new(
-                context,
-                &"
-                uniform sampler2DArray colorMap;
-                uniform sampler2DArray depthMap;
-                uniform int colorLayer;
-                uniform int depthLayer;
-                in vec2 uv;
-                layout (location = 0) out vec4 color;
-                void main()
-                {
-                    color = texture(colorMap, vec3(uv, colorLayer));
-                    gl_FragDepth = texture(depthMap, vec3(uv, depthLayer)).r;
-                }",
-            )?);
-        }
-        Ok(COPY_EFFECT.as_ref().unwrap())
-    }
-}
