@@ -22,7 +22,11 @@ impl OrbitControl {
         }
     }
 
-    pub fn handle_events(&mut self, camera: &mut Camera, events: &mut [Event]) -> Result<bool> {
+    pub fn handle_events(
+        &mut self,
+        camera: &mut Camera,
+        events: &mut [Event],
+    ) -> ThreeDResult<bool> {
         if let CameraAction::Zoom { speed, target, .. } = &mut self.control.scroll_horizontal {
             *speed = 0.1 / target.distance(*camera.position());
         }
